# encoding: utf-8

"""

Base classes for implementing stories using behavior driven development
(this is not a portage of jbehave but a kind of python counterpart).
"""

import pybehave.utils

__author__ = 'Raphael.Marvie@lifl.fr'
__date__ = '2007-03-20'


class Renderable(object):
    """Can be rendered using a Renderer"""

    def narrate_to(self, renderer):
        renderer.render_any(self)
        

class Story(Renderable):
    """Base class for implementing a scenario driven Story."""
    
    def __init__(self, narrative=None, scenarios=None, output=None):
        self.narrative = narrative
        if not scenarios:
            scenarios = list()
        self.scenarios = scenarios
        self.output = output
        
    def add_scenario(self, scenario):
        """Add a scenario to the story"""
        scenario.specify()
        self.scenarios.append(scenario)
        
    def specify(self):
        """Specify the scenarios of the story."""
        pass

    def run(self):
        """Run the various scenarios of the story."""
        for scenario in self.scenarios:
            self.output.process(self._run_scenario(scenario, dict()))
    
    def _run_scenario(self, scenario, world):
        story_name = pybehave.utils.make_phrase(self.__class__.__name__)
        scenario_name = pybehave.utils.make_phrase(scenario.__class__.__name__)
        try:
            scenario.run(world)
            return pybehave.result.ScenarioResult(story_name, scenario_name, 
                                                  "SUCCESS")
        except AssertionError, ae:
            return pybehave.result.ScenarioResult(story_name, scenario_name, 
                                                  "FAILED", str(ae))
    
    def __repr__(self):
        result = list()
        result.append('Story %s' % \
                      pybehave.utils.make_phrase(self.__class__.__name__))
        if self.narrative:
            result.append(str(self.narrative))
        result.append('')
        for scenario in self.scenarios:
            result.append(str(scenario))
        return '\n'.join(result)


class Scenario(Renderable):
    """Base class for implementing a Scenario."""
    
    def __init__(self, context=None, events=None, outcomes=None):
        if not events:
            events = list()
        self.events = events
        if not outcomes:
            outcomes = list()
        self.outcomes = outcomes
        self.steps = list()
        
    def given(self, context):
        """Add the context to the scenario."""
        self.steps.append(context)
        
    def when(self, event):
        """Add an event to the scenario."""
        self.steps.append(event)

    def then(self, outcome):
        """Add an outcome to the scenario."""
        self.steps.append(outcome)
        
    def run(self, world):
        """Run the scenario in the provided world."""
        for step in self.steps:
            step.perform(world)

    def __repr__(self):
        result = list()
        result.append('Scenario %s' % \
            pybehave.utils.make_phrase(self.__class__.__name__))
        if self.steps:
            previous = self.steps[0]
            
        result.append('')
        return '\n'.join(result)


class Given(object):
    """Base class for defining a Given."""
    
    def perform(self, world):
        """Perform the step in the provided world (dict)."""
        self.setup(world)

    def setup(self, world):
        """Set up the state of the provided world (dict)."""
        pass
        
    def cleanup(self, world):
        """Clean up the actions of this given on the world."""
        pass
        

class Event(object):
    """Base class for defining an Event (when)."""
    
    def perform(self, world):
        """Perform the step in the provided world (dict)."""
        self.occur_in(world)

    def occur_in(self, world):
        """Make this event occur in the provided world (dict)."""
        pass

    def cleanup(self, world):
        """Clean up the actions of this event on the world."""
        pass


class Outcome(object):
    """Base class for defining an Outcome (ensure)"""
    
    def perform(self, world):
        """Perform the step in the provided world (dict)."""
        self.verify(world)

    def verify(self, world):
        """Verify the state of the provided world (dict)."""
        pass

    def cleanup(self, world):
        """Clean up the actions of this outcome on the world."""
        pass


class Narrative(object):
    """Definition of a Narrative to provide information about the story."""

    def __init__(self, role=None, feature=None, benefit=None):
        self.role = role
        self.feature = feature
        self.benefit = benefit
        
    def __repr__(self):
        return '\n'.join([
            'As a %s' % self.role,
            'I want to %s' % self.feature,
            'Such as %s' % self.benefit,
        ])

# eof
