# encoding: utf-8

""" 
Generators for producing tests skeletons (unit tests or pybehave ones) from
the model of a story defined using pybehave.model.

    >>> from pybehave.model import *
    >>> context1 = Given('a user of group')
    >>> event1 = Event('logged in')
    >>> event2 = Event('has task todo')
    >>> ensure1 = Ensure('user can view invoices')
    >>> ensure2 = Ensure('user can read task details')
    >>> story = Story('My First story',
    ...     role='User',
    ...     feature='to be abble to work',
    ...     benefit='I receive a salary',
    ...     scenarios=[Scenario('Happy user', [context1], [event1], [ensure1]),
    ...      Scenario('User has to work', [context1], [event2], [ensure2]),
    ...     ]
    ... )
    ... 


Generate a unit test skeleton

    >>> print generate_unit_tests(story)
    import unittest
    class TestAUserOfGroup(unittest.TestCase):
        def test_when_logged_in_user_can_view_invoices(self): pass
        def test_when_has_task_todo_user_can_read_task_details(self): pass
    def suite():
        suite = unittest.TestSuite()
        suite.addTest(unittest.makeSuite(TestAUserOfGroup)
        return suite
    if __name__ == '__main__': unittest.main()
    
    
Generate a BDD Story module
    
    >>> print generate_bdd_story(story)
    # encoding: utf-8
    # generated by pybehave (do not edit)
    <BLANKLINE>
    import pybehave.core
    import pybehave.result
    from my_first_story_skels import *
    <BLANKLINE>
    class MyFirstStory(pybehave.core.Story):
        def __init__(self):
            super(MyFirstStory, self).__init__(
                pybehave.core.Narrative("User", "to be abble to work", "I receive a salary")
            )
        def specify(self):
            self.add_scenario(HappyUser())
            self.add_scenario(UserHasToWork())
    <BLANKLINE>
    class HappyUser(pybehave.core.Scenario):
        def __init__(self):
            super(HappyUser, self).__init__()
        def specify(self):
            self.given(AUserOfGroup())
            self.when(LoggedIn())
            self.then(UserCanViewInvoices())
    <BLANKLINE>
    class UserHasToWork(pybehave.core.Scenario):
        def __init__(self):
            super(UserHasToWork, self).__init__()
        def specify(self):
            self.given(AUserOfGroup())
            self.when(HasTaskTodo())
            self.then(UserCanReadTaskDetails())
    <BLANKLINE>
    def main(output):
        story = MyFirstStory()
        story.output = output
        story.specify()
        story.run()
    <BLANKLINE>
    if __name__ == '__main__':
        main(pybehave.result.SimpleScenarioOutput())
    <BLANKLINE>
    #eof

Generate a BDD skeleton module
    
    >>> print generate_bdd_skels(story)
    # encoding: utf-8
    # generated by pybehave (to be completed)
    <BLANKLINE>
    import pybehave.core
    <BLANKLINE>
    class AUserOfGroup(pybehave.core.Given):
        def setup(self, world): pass
    <BLANKLINE>
    class LoggedIn(pybehave.core.Event):
        def occur_in(self, world): pass
    <BLANKLINE>
    class HasTaskTodo(pybehave.core.Event):
        def occur_in(self, world): pass
    <BLANKLINE>
    class UserCanViewInvoices(pybehave.core.Outcome):
        def verify(self, world): pass
    <BLANKLINE>
    class UserCanReadTaskDetails(pybehave.core.Outcome):
        def verify(self, world): pass
    <BLANKLINE>
    #eof
"""


import pybehave.model
import pybehave.utils


__author__ = 'Raphael.Marvie@lifl.fr'
__date__ = '2007-03-20'


######################################################################
# Generator of unit test skeletons from story's specification
######################################################################

def generate_unit_tests(story):
    """Return the unit test skeleton derived from the story definition."""
    tests = dict()
    for scenario in story.scenarios:
        if len(scenario.given) == 1 and \
           len(scenario.when) == 1 and \
           len(scenario.ensure) == 1:
            name = pybehave.utils.make_camel_name(scenario.given[0].desc)
            try:
                test = tests[name]
            except KeyError:
                test = pybehave.model.TestCase(name)
                tests[name] = test
            fct_name = 'when %s %s' % (scenario.when[0].desc, 
                                       scenario.ensure[0].desc)
            test.functions.append(pybehave.utils.make_python_name(fct_name))
    return str(pybehave.model.TestSuite(tests.values()))


######################################################################
# Generator of BDD stories and skeletons from story's specification
######################################################################

class GenerateBddStory(object):
        
    def _generate_header(self,story):
        self.result.append('# encoding: utf-8')
        self.result.append('# generated by pybehave (do not edit)\n')
        self.result.append('import pybehave.core')
        self.result.append('import pybehave.result')
        self.result.append('from %s_skels import *\n' % \
            pybehave.utils.make_python_name(story.name))        
        
    def _generate_footer(self, story):
        self.result.append('def main(output):')
        self.result.append('    story = %s()' % \
            pybehave.utils.make_camel_name(story.name))            
        self.result.append('    story.output = output')
        self.result.append('    story.specify()')
        self.result.append('    story.run()')
        self.result.append('')
        self.result.append("if __name__ == '__main__':")
        self.result.append('    main(pybehave.result.SimpleScenarioOutput())')
        self.result.append('')
        self.result.append('#eof')
        
    def _generate_story(self, story):
        story_name = pybehave.utils.make_camel_name(story.name)
        self.result.append('class %s(pybehave.core.Story):' % story_name)
        self.result.append('    def __init__(self):')
        self.result.append('        super(%s, self).__init__(' % story_name)
        if story.role and story.feature and story.benefit:
            self.result.append('            pybehave.core.Narrative("%s", "%s", "%s")' % \
                (story.role, story.feature, story.benefit))
        self.result.append('        )')
        self.result.append('    def specify(self):')
        for scenario in story.scenarios:
            self.result.append('        self.add_scenario(%s())' % \
                pybehave.utils.make_camel_name(scenario.name))
        self.result.append('')
        
    def _generate_scenarios(self, story):
        for scenario in story.scenarios:
            scenario_name = pybehave.utils.make_camel_name(scenario.name)
            self.result.append('class %s(pybehave.core.Scenario):' % \
                scenario_name)
            self.result.append('    def __init__(self):')
            self.result.append('        super(%s, self).__init__()' % \
                scenario_name)
            self.result.append('    def specify(self):')
            for given in scenario.given:
                self.result.append('        self.given(%s())' % \
                    pybehave.utils.make_camel_name(given.desc))
            for when in scenario.when:
                self.result.append('        self.when(%s())' % \
                    pybehave.utils.make_camel_name(when.desc))
            for ensure in scenario.ensure:
                self.result.append('        self.then(%s())' % \
                    pybehave.utils.make_camel_name(ensure.desc))
            self.result.append('')
        
    def __call__(self, story):
        self.result = list()
        self._generate_header(story)
        self._generate_story(story)
        self._generate_scenarios(story)
        self._generate_footer(story)        
        return str('\n'.join(self.result))
        

def generate_bdd_story(story):
    """Return the BDD story derived from the story definition."""
    return GenerateBddStory()(story)


class GenerateBddSkels(object):
        
    def _generate_header(self):
        self.result.append('# encoding: utf-8')
        self.result.append('# generated by pybehave (to be completed)\n')
        self.result.append('import pybehave.core\n')
        
    def _generate_items(self, story, name, clss, fct):
        items = list()
        for scenario in story.scenarios:
            for item in getattr(scenario, name):
                if item not in items:
                    items.append(item)
        for item in items:
            self.result.append('class %s(pybehave.core.%s):' % \
                (pybehave.utils.make_camel_name(item.desc), clss))
            self.result.append('    def %s(self, world): pass' % fct)
            self.result.append('')
        
    def _generate_givens(self, story):
        self._generate_items(story, 'given', 'Given', 'setup')

    def _generate_events(self, story):
        self._generate_items(story, 'when', 'Event', 'occur_in')

    def _generate_outcomes(self, story):
        self._generate_items(story, 'ensure', 'Outcome', 'verify')
       
    def _generate_footer(self):
        self.result.append('#eof')
        
    def __call__(self, story):
        self.result = list()
        self._generate_header()
        self._generate_givens(story)
        self._generate_events(story)
        self._generate_outcomes(story)
        self._generate_footer()        
        return str('\n'.join(self.result))


def generate_bdd_skels(story):
    """Return the BDD test skeleton derived from the story definition."""
    return GenerateBddSkels()(story)


def generate_bdd(story, with_skeletons=False):
    module_name = pybehave.utils.make_python_name(story.name)
    output = open('%s.py' % module_name, 'w')
    output.write(generate_bdd_story(story))
    output.close()
    if with_skeletons:
        output = open('%s_skels.py' % module_name, 'w')
        output.write(generate_bdd_skels(story))
        output.close()
        

if __name__ == '__main__':
    import doctest
    doctest.testmod()
    
# eof
