# encoding: utf-8

""" 
Classes for creating internal representation of story and unittest
Test Suites.

Definition of a story model

    >>> context1 = Given('a user of group')
    >>> context2 = Given('a manager of group')
    >>> event1 = Event('logged in')
    >>> event2 = Event('has task todo')
    >>> ensure1 = Ensure('user can view invoices')
    >>> ensure2 = Ensure('user can read task details')
    >>> ensure3 = Ensure('manager can assign tasks to users')
    >>> spec = Story('First story',
    ...     scenarios=[Scenario('1', [context1], [event1], [ensure1]),
    ...      Scenario('2', [context1], [event2], [ensure2]),
    ...      Scenario('3', [context2], [event1], [ensure3]),
    ...     ]
    ... )
    ... 
    >>> print spec
    Story: First story
    <BLANKLINE>
    Scenario: 1
    <BLANKLINE>
    Given a user of group
    When logged in
    Ensure user can view invoices
    <BLANKLINE>
    Scenario: 2
    <BLANKLINE>
    Given a user of group
    When has task todo
    Ensure user can read task details
    <BLANKLINE>
    Scenario: 3
    <BLANKLINE>
    Given a manager of group
    When logged in
    Ensure manager can assign tasks to users
    <BLANKLINE>


Definition of a Test Suite model

    >>> suite = TestSuite()
    >>> test1 = TestCase('GivenAUserOfGroup')
    >>> test1.functions.append('when_logged_in_user_can_view_invoices')
    >>> test1.functions.append('when_has_task_todo_user_can_read_task_details')
    >>> suite.tests.append(test1)
    >>> test2 = TestCase('AManagerOfGroup')
    >>> test2.functions.append('when_logged_in_manager_can_assign_tasks_to_users')
    >>> suite.tests.append(test2)
    >>> print suite
    import unittest
    class TestGivenAUserOfGroup(unittest.TestCase):
        def test_when_logged_in_user_can_view_invoices(self): pass
        def test_when_has_task_todo_user_can_read_task_details(self): pass
    class TestAManagerOfGroup(unittest.TestCase):
        def test_when_logged_in_manager_can_assign_tasks_to_users(self): pass
    def suite():
        suite = unittest.TestSuite()
        suite.addTest(unittest.makeSuite(TestGivenAUserOfGroup)
        suite.addTest(unittest.makeSuite(TestAManagerOfGroup)
        return suite
    if __name__ == '__main__': unittest.main()
"""


__author__ = 'Raphael.Marvie@lifl.fr'
__date__ = '2007-03-20'


######################################################################
# Classes for modeling a specification
######################################################################

class Specification(object):
    """Representation for a specification definition."""
    def __init__(self, stories=None):
        if not stories:
            stories = list()
        self.stories = stories

    def __repr__(self):
        res = list()
        for story in self.stories:
            res.append(str(story))
        return '\n'.join(res)
        

class Story(object):
    """Representation for a story definition."""
    
    def __init__(self, name=None, role=None, feature=None, benefit=None, 
                 scenarios=None):
        self.name = name
        self.role = role
        self.feature = feature
        self.benefit = benefit
        if not scenarios:
            scenarios = list()
        self.scenarios = scenarios
        
    def __repr__(self):
        res = list()
        res.append('Story: %s' % self.name)
        if self.role and self.feature and self.benefit:
            res.append('')
            res.append('As a %s' % self.role)
            res.append('I want %s' % self.feature)
            res.append('So that %s' % self.benefit)
        res.append('')
        for scenario in self.scenarios:
            res.append(str(scenario))
        return '\n'.join(res)


class Scenario(object):
    """Representation for a scenario definition."""

    def __init__(self, name=None, given=None, when=None, ensure=None):
        self.name = name
        if not given:
            given = list()
        self.given = given
        if not when:
            when = list()
        self.when = when
        if not ensure:
            ensure = list()
        self.ensure = ensure
        
    def __repr__(self):
        return '\n'.join([
            'Scenario: %s' % self.name,
            '',
            'Given %s' % '\nand '.join([x.desc for x in self.given]),
            'When %s' % '\nand '.join([x.desc for x in self.when]),
            'Ensure %s' % '\nand '.join([x.desc for x in self.ensure]),
            ''
        ])


class Given(object):
    """Representation for a context definition."""
    
    def __init__(self, desc):
        self.desc = desc


class Event(object):
    """Representation for an event definition."""
    
    def __init__(self, desc):
        self.desc = desc
        

class Ensure(object):
    """Representation for an ensure definition."""
    def __init__(self, desc):
        self.desc = desc

                    
######################################################################
# Classes for modeling a test suite
######################################################################

class TestSuite(object):
    """Representation for a test suite definition."""
    
    def __init__(self, tests=None):
        if not tests:
            tests = list()
        self.tests = tests
        
    def __repr__(self):
        """docstring for __repr__"""
        res = list()
        res.append('import unittest')
        for test in self.tests:
            res.append(str(test))
        res.append('def suite():')
        res.append('    suite = unittest.TestSuite()')
        for test in self.tests:
            res.append('    suite.addTest(unittest.makeSuite(Test%s)' % \
                       test.name)
        res.append('    return suite')
        res.append("if __name__ == '__main__': unittest.main()")
        return '\n'.join(res)


class TestCase(object):
    """Representation for a test case definition."""

    def __init__(self, name, functions=None):
        self.name = name
        if not functions:
            functions = list()
        self.functions = functions
        
    def __repr__(self):
        res = list()
        res.append('class Test%s(unittest.TestCase):' % self.name)
        for function in self.functions:
            res.append('    def test_%s(self): pass' % function)
        return '\n'.join(res)


if __name__ == '__main__':
    import doctest
    doctest.testmod()

# eof
