# encoding: utf-8

""" 
Parser of textual story that produces an internal representation
using pybehave.model. 

    >>> spec = '''
    ... Story: A story
    ... 
    ... As a User
    ... I want to be abble to work
    ... So that I receive a salary
    ... 
    ... Scenario: First Scenario
    ... 
    ... Given a user of group
    ... When logged in
    ... Ensure user can view tasks
    ... 
    ... Scenario: Second Scenario
    ... 
    ... Given a user of group
    ... When logged in
    ...  and has task todo
    ... Ensure user can read task details
    ... '''
    >>> parse = StoryParser()
    >>> print parse(spec)
    Story: A story
    <BLANKLINE>
    As a User
    I want to be abble to work
    So that I receive a salary
    <BLANKLINE>
    Scenario: First Scenario
    <BLANKLINE>
    Given a user of group
    When logged in
    Ensure user can view tasks
    <BLANKLINE>
    Scenario: Second Scenario
    <BLANKLINE>
    Given a user of group
    When logged in
    and has task todo
    Ensure user can read task details
    <BLANKLINE>
"""

import pybehave.model

__author__ = 'Raphael.Marvie@lifl.fr'
__date__ = '2007-03-20'


class StoryParser(object):
    """Trivial parser for textual story definitions.
    
    The textual story has to match the following template:
    
    Story: A story
    
    As a User 
    I want to be abble to work
    So that I receive a salary
    
    Scenario: First scenario
    
    Given a user of group
    When logged in
    Ensure user can view invoices
    
    Scenario: Second scenario
    
    Given a user of group
    When logged in
    and has task todo
    Ensure user can read task details
    
    etc.
    """
    
    def _initialize(self, tstory):
        self.lines = tstory.split('\n')
        self.current = 0
        self.spec = pybehave.model.Specification()
        self.given = dict()
        self.when = dict()
        self.ensure = dict()
    
    def __call__(self, story):
        self._initialize(story)
        line = self._get_current_line()
        while line is not None and not line.startswith('Story:'):
            line = self._get_next_line()
        if self.current < len(self.lines):
            self.spec.stories.append(self._parse_story())
        return self.spec
        
    def _get_current_line(self):
        if self.current < len(self.lines):
            return self.lines[self.current].strip()

    def _get_next_line(self):
        self.current += 1
        line =  self._get_current_line()
        while line is not None and not line.strip():
            self.current += 1
            line =  self._get_current_line()            
        return line
        
    def _parse_story(self):
        name = self._get_current_line()[len('Story:'):].strip()
        self._get_next_line()
        role = self._parse_role()
        feature = self._parse_feature()
        benefit = self._parse_benefit()
        story = pybehave.model.Story(name, role, feature, benefit)
        line = self._get_current_line()
        while line is not None:
            while line is not None and not line.startswith('Scenario:'):
                line = self._get_next_line()
            if self.current < len(self.lines):
                story.scenarios.append(self._parse_scenario())
            else:
                break
            line = self._get_current_line()
        return story
        
    def _parse_role(self):
        return self._parse_line('As a')

    def _parse_feature(self):
        return self._parse_line('I want')

    def _parse_benefit(self):
        return self._parse_line('So that')

    def _parse_line(self, item):
        line = self._get_current_line()
        if line and line.startswith(item):
            res = line[len(item):].strip()
            self._get_next_line()
            return res

    def _parse_scenario(self):
        line = self._get_current_line()
        assert line.startswith('Scenario:'), \
               "Expected <Scenario> declaration line %d not found" % \
               (self.current + 1,)
        name = line[len('Scenario:'):].strip()
        self._get_next_line()
        given = self._parse_given()
        when = self._parse_when()
        ensure = self._parse_ensure()
        return pybehave.model.Scenario(name, given, when, ensure)

    def _parse_given(self):
        return self._parse_item('Given', pybehave.model.Given, self.given)

    def _parse_when(self):
        return self._parse_item('When', pybehave.model.Event, self.when)

    def _parse_ensure(self):
        return self._parse_item('Ensure', pybehave.model.Ensure, self.ensure)

    def _parse_item(self, item, clss, index):
        assert self.current < len(self.lines), \
            "EOF encountered while expecting <%s> line %d" % \
            (item, self.current + 1)
        line = self._parse_line(item)
        assert line, "Expected <%s> declaration line %d not found" % \
               (item, self.current + 1) 
        items = list()
        while line is not None:
            items.append(self._add_to_index(line.strip(), clss, index))
            line = self._parse_line('and')            
        return items
        
    def _add_to_index(self, desc, clss, index):
        if not desc in index:
            result = clss(desc)
            index[desc] = result
        else:
            result = index[desc]
        return result
        

if __name__ == '__main__':
    import doctest
    doctest.testmod()
    
# eof
